# -*-python-*-

__package__    = "turkishMMap/Providers/__init__.py"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/turkishMMap"
__date__       = "$Date: 2009/05/09 17:04:00 $"
__copyright__  = "Copyright (c) 2009 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import turkishMMap

from modestMMarkers import utils
from modestMMarkers import primitives
from modestMMarkers.feeds import mmGeoRSS
from modestMMarkers.feeds import mmAtom

import PIL.Image
import StringIO

#
# Base 'feed' provider class - this still needs to be refactored
# (where possible) into modestMMarkers.feeds (or whatever that becomes)
#

class tmProvider (turkishMMap.turkishMMap) :
        
    def __init__ (self, *args) :
        turkishMMap.turkishMMap.__init__(self, *args)
        self.feed = None
        self.qr_url = None
        
    ##############################################################
        
    def get_feed(self, url) :
        xml = self.fetch_url(url)
        self.feed = self.parse_feed(xml)
        return self.feed
        
    ##############################################################

    def draw_feed_paginated_orphans(self, bbox, orphans) :

        self.reset()
        self.add_markers(orphans)
        self.page_title = '%s : oddballs and orphans' % self.feed['title']
        
        self.setup(bbox)
        self.draw_map(bbox)
        self.draw_markers()
        self.draw_canvas()
        self.draw_legend()
        self.draw_page_title()
        
        self.pages.append(self.canvas)        

    ##############################################################
        
    def draw_feed_paginated_splash_page(self, bbox, boxes, markers, qr_url=None) :

        self.reset()
        self.draw_fold_lines = False
        self.page_title = self.feed['title']
        
        self.add_boxes(boxes)
        
        self.setup(bbox)
        self.draw_map(bbox)
        self.draw_boxes()
        self.draw_markers()
        
        surface = utils.pil2cairo(self.mm_img)

        for m in markers :

            p = m[0]
            c = m[1]

            pt = utils.latlon_to_point(self.mm_obj, c.x, c.y)
            args = {'x': pt.x, 'y': pt.y, 'string' : p, 'radius':10}

            primitives.draw_circle_with_string(surface, args)

        self.mm_img = utils.cairo2pil(surface)

        self.draw_canvas()
        
        #
        # qr code (experimental, still)
        #

        if qr_url :

            try :
                (w, h) = self.mm_img.size
            
                gd_url = 'http://is.gd/api.php?longurl=%s' % qr_url
                qr_txt = self.fetch_url(gd_url)

                # print qr_txt
                
                qr_w = int(self.min_dim / 5)
                
                url = 'http://chart.apis.google.com/chart?cht=qr&chs=%sx%s&chl=%s&chld=H|0' % (qr_w, qr_w, qr_txt)
                src = self.fetch_url(url)
                
                im = PIL.Image.open(StringIO.StringIO(src)).convert('RGBA')
                
                x_off = self.width - qr_w - self.margin
                y_off = self.margin
                
                self.canvas.paste(im, (x_off, y_off))
            except Exception, e:
                print "QR wrangling failed, %s" % e
                
        #
        # carry on
        #
        
        self.draw_legend()
        self.draw_page_title()
        
        self.pages.append(self.canvas)        

    ##############################################################
    
    def draw_feed_paginated(self, url, **kwargs) :

        zoom = 15
        k = None

        if kwargs.has_key('zoom') :
            zoom = kwargs['zoom']

        if kwargs.has_key('k') :
            k = kwargs['k']
            
        try :
            import clusterMMap
        except Exception, e :
            print e
            return False

        #
        # mmmm.....data
        #

        feed = self.get_feed(url)
        
        #
        # cluster!
        #

        cl = clusterMMap.clusterMMap()
        (clusters, abandoned) = cl.clusters(feed['points'], k)

        paginated = cl.paginate(clusters, feed['points'], feed['items'], feed['lookup'])

        #
        # title page
        #
        
        boxes = []
        markers = []
        orphans = []

        pts = utils.mk_points_dictify(feed['points'], True)
        bbox = utils.calculate_bbox_for_points(pts)

        bbox_points = ({'latitude':bbox[2], 'longitude':bbox[1]},
                       {'latitude':bbox[0], 'longitude':bbox[1]},
                       {'latitude':bbox[0], 'longitude':bbox[3]},
                       {'latitude':bbox[2], 'longitude':bbox[3]},
                       {'latitude':bbox[2], 'longitude':bbox[1]})
        
        boxes.append(bbox_points)
        
        i = 1
        
        for poly in clusters :

            c = poly.centroid
            markers.append((i, c))
                           
            box = cl.poly2box(poly)
            coords = list(box.exterior.coords)
            coords = utils.mk_points_dictify(coords)
            boxes.append(coords)

            i += 1
            
        self.draw_feed_paginated_splash_page(bbox, boxes, markers, self.qr_url)

        #
        # the individual maps
        #
        
        i = 1
        
        for pg in paginated :
                
                ctx = cl.poly2box(pg['poly'])
                ctx = list(ctx.exterior.coords)
                ctx = utils.mk_points_dictify(ctx)

                box = utils.expand_bbox(pg['poly'].bounds, .1)

                self.reset()
                
                self.draw_fold_lines = True
                self.draw_navi_graphic = True
                self.navi_graphic_bbox = box
                self.navi_graphic_context = ctx

                self.page_title = "%s : map %s" % (self.feed['title'], i)
                
                self.add_markers(pg['markers'])

                # FIX ME : bbox needs to be calculate based on 
                # pg['markers'] and all the anchor/overlap offsets
                
                self.draw(utils.expand_bbox(pg['poly'].bounds, .01))
                i += 1

        #
        # draw the orphans
        #
        
        for pt in abandoned :

            key = "%s,%s" % (pt[0], pt[1])
            uid = feed['lookup'][key]
            orphans.append(feed['items'][uid])

        self.draw_feed_paginated_orphans(bbox, orphans)
        
        # happy
        
        return True

#
# Atom
#

class Atom (tmProvider, mmAtom.mmAtom) :

    def __init__ (self, *args) :

        tmProvider.__init__(self, *args)
        mmAtom.mmAtom.__init__(self, *args)

    # this is kind of fucked up really...
    
    def draw_feed (self, url, **kwargs) :

        if kwargs.has_key('paginated') :
            return self.draw_feed_paginated(url, **kwargs)

        return mmAtom.mmAtom.draw_feed(self, url, **kwargs)

#
# GeoRSS
#

class GeoRSS (tmProvider, mmGeoRSS.mmGeoRSS) :

    def __init__ (self, *args) :

        tmProvider.__init__(self, *args)
        mmGeoRSS.mmGeoRSS.__init__(self, *args)    

    # see above
    
    def draw_feed (self, url, **kwargs) :

        if kwargs.has_key('paginated') :
            return self.draw_feed_paginated(url, **kwargs)

        return mmGeoRSS.mmGeoRSS.draw_feed(self, url, **kwargs)
